#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/cdrom.h>
#include <stdio.h>
#include <string.h>
#include "jac_cmd.h" 
#include "jac_lang.h"
#include "jac_file.h"
#include "mixer.h"     


int cd_open (const char *device,int opt)
{
	int cd_fd;
	if ((cd_fd = open(device, O_RDONLY)) == -1 && !opt) {
		fprintf(stderr,APRENDO,device);
		exit (1);
	}
	if (opt && cd_fd==-1)
		cd_fd=0;

	return cd_fd;
}

int cd_close(int cd_fd)
{
	if (close(cd_fd)!=0) {
		fprintf(stderr,CHIUDENDO);
		exit(1);
	}
	return 0;
}

cd *cd_init(int cd_fd)
{
	cd *hw = (cd *) malloc(sizeof(cd));
	int i;
	
 	/* Read Header */
	if ( ioctl(cd_fd, CDROMREADTOCHDR, &(hw->tochdr)) == -1 ) {
		fprintf(stderr,ERRTOCHDR);   
		exit(1);
	}

	/* Read Tracks */            
	for (i=hw->tochdr.cdth_trk0; i<=hw->tochdr.cdth_trk1; i++) {
		hw->tocent[i-1].cdte_track = i;
		hw->tocent[i-1].cdte_format = CDROM_MSF;
		if ( ioctl(cd_fd, CDROMREADTOCENTRY, &(hw->tocent[i-1])) == -1 ) {
		  fprintf(stderr,ERRTOCHDR);   
			exit(1);
		}
	}
	hw->tocent[hw->tochdr.cdth_trk1].cdte_track = CDROM_LEADOUT;
	hw->tocent[hw->tochdr.cdth_trk1].cdte_format = CDROM_MSF;
	if ( ioctl(cd_fd, CDROMREADTOCENTRY, &(hw->tocent[hw->tochdr.cdth_trk1])) == -1 ) {
		fprintf(stderr,ERRTOCHDR);   
		exit(1);
	}

	/* Read Subchannel Info */
	hw->subchnl.cdsc_format = CDROM_MSF;
	if ( ioctl(cd_fd, CDROMSUBCHNL, &(hw->subchnl)) == -1 ) {
		fprintf(stderr,ERRSUBCHNL);   
		exit(1);
	}

	return hw;
}

int first_track(int cd_fd) 
{
	cd *hw = cd_init(cd_fd);
	return (hw->tochdr.cdth_trk0);
}

int last_track(int cd_fd) 
{
	cd *hw = cd_init(cd_fd);
	return (hw->tochdr.cdth_trk1);
}

/* Play */
void cd_play (int cd_fd, int trk) 
{
	struct cdrom_msf msf;
	cd *hw=cd_init(cd_fd);

	msf.cdmsf_min0=hw->tocent[trk-1].cdte_addr.msf.minute;
	msf.cdmsf_sec0=hw->tocent[trk-1].cdte_addr.msf.second;
	msf.cdmsf_frame0=hw->tocent[trk-1].cdte_addr.msf.frame;
	msf.cdmsf_min1=hw->tocent[trk].cdte_addr.msf.minute;
	msf.cdmsf_sec1=hw->tocent[trk].cdte_addr.msf.second;
	msf.cdmsf_frame1=hw->tocent[trk].cdte_addr.msf.frame;

 	if ( ioctl(cd_fd, CDROMPLAYMSF, &msf) == -1 ) {
		fprintf(stderr,ERRPLAY);   
		perror("play(): ");
		exit(1);
	} 
}


int crazyplay(int cd_fd,int arr_track[MAXTRACK],int ncrazy)
{
	struct cdrom_msf msf;
	int nsectot,first,last,repeat,crazy,min,sec,nrnd,i,wait;
	int secini,secfin,sectemp,trstart,trstop,crdata;
	cd *hw=cd_init(cd_fd);
	crdata=0;

	first=hw->tochdr.cdth_trk0;
	last=hw->tochdr.cdth_trk1;
	nsectot=(hw->tocent[last].cdte_addr.msf.minute*60) +
				hw->tocent[last].cdte_addr.msf.second;
	if (ncrazy>nsectot)
		ncrazy=nsectot-1;
	nsectot-=ncrazy;
	arr_track[0]=0;
	crazy=1;

	/* If CD is playing I stop it and start playing myself */
	if (cd_status(cd_fd)==CDROM_AUDIO_PLAY) {
		writedata(cd_fd,0,arr_track,0,9);
		cd_stop(cd_fd);
		sleep(3);
	}

	srand((int) getpid());
	writedata(cd_fd,0,arr_track,0,crazy);
	while (crazy!=9) {
		crdata=0;
		nrnd=rand() % nsectot;
		min=nrnd / 60;
		sec=nrnd % 60;
		msf.cdmsf_min0=min;
		msf.cdmsf_sec0=sec;
		msf.cdmsf_frame0=1;
		sec+=ncrazy;
		if (sec>59) {
			sec-=60;
			min++;
		}
		msf.cdmsf_min1=min;
		msf.cdmsf_sec1=sec;
		msf.cdmsf_frame1=1;

		/* Check for DATA TRACK inside crazy piece of disk */

		/* Absolute seconds of crazy playing start */
		secini=msf.cdmsf_min0 * 60 + msf.cdmsf_sec0;
		/* Absolute seconds of crazy playing stop */
		secfin=msf.cdmsf_min1 * 60 + msf.cdmsf_sec1;
		trstart=-1;
		trstop=-1;
		for (i=first;i<=last;i++) {
			/* Absolute seconds of track #i start */
			sectemp=hw->tocent[i].cdte_addr.msf.minute * 60 + hw->tocent[i].cdte_addr.msf.second;
			if (sectemp>=secini && trstart==-1)
				trstart=i;
			if (sectemp>=secfin && trstop==-1)
				trstop=i;
		}
		for (i=trstart;i<=trstop;i++) {
			if (isdata(cd_fd,i)) {
				crdata=1;
			  break;   
			}
		}
		if (crdata)
			continue;
	
		/* End of DATA TRACK search */

 		if ( ioctl(cd_fd, CDROMPLAYMSF, &msf) == -1 ) {
			fprintf(stderr,ERRPLAY);   
			perror("crazyplay(): ");
			exit(1);
		} 
		wait=1;
		while (wait) {
		 	switch (cd_status(cd_fd)) {
				case CDROM_AUDIO_PLAY:
				case CDROM_AUDIO_PAUSED:
					sleep(2);
					break;
				default:
					i=gettracktoplay(FILEPARM,arr_track,last,&repeat,&crazy); 
					wait=0;
					break;
			}
		} 
	}
	return 0;	
}


void cd_stop(int cd_fd) 
{
	if ( ioctl(cd_fd, CDROMSTOP) == -1) {
		fprintf(stderr,ERRSTOP);   
		exit(1);
	}
}

void cd_pause(int cd_fd) 
{
	if ( ioctl(cd_fd, CDROMPAUSE) == -1) {
		fprintf(stderr,ERRPAUSE);   
		exit(1);
	}
}

void cd_resume(int cd_fd) 
{
	if ( ioctl(cd_fd, CDROMRESUME) == -1) {
		fprintf(stderr,ERRRESUME);   
		exit(1);
	}
}

void cd_eject(int cd_fd) 
{
	if ( ioctl(cd_fd, CDROMEJECT) == -1) {
		fprintf(stderr,ERREJECT);   
		exit(1);
	}
}

int cd_status (int cd_fd) 
{
	int stato;
	cd *hw = cd_init(cd_fd);
	stato=hw->subchnl.cdsc_audiostatus;
	free(hw);
	return stato;
}

int isdata(int cd_fd,int track) 
{
	int ret=0;
	cd *hw = cd_init(cd_fd);
	if (hw->tocent[track-1].cdte_ctrl==CDROM_DATA_TRACK)
		ret=1;	
	free(hw);
	return ret;
}

int is_playing (int cd_fd)
{
	cd *hw = cd_init(cd_fd);
	if (hw->subchnl.cdsc_audiostatus == CDROM_AUDIO_PLAY)
		return (hw->subchnl.cdsc_trk);
	return 0;
}

int is_stopped (int cd_fd)
{
	return (cd_status(cd_fd) == CDROM_AUDIO_PLAY);
}

int is_paused (int cd_fd)
{
	return (cd_status(cd_fd) == CDROM_AUDIO_NO_STATUS);
}
