#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h> 
#include <stdio.h> 
#include "mixer.h" 
#include "jac_cmd.h"
#include "jac_file.h"
#include "jac.h"
#include "jac_cfg.h"
#include "jac_lang.h"
#include "jac_cddb.h"
#include "jac_console.h"


void show_version() 
{
	fprintf(stdout,"jac version %s\n",VERSION);
	fprintf(stdout,"(C)opyleft Cosimo Vagarini 1999-2000\n\n");
	fprintf(stdout,"This program is free software; you can redistribute it and/or modify\n");
	fprintf(stdout,"it under the terms of the GNU General Public License as published by\n");
	fprintf(stdout,"the Free Software Foundation; either version 2 of the License, or\n");
	fprintf(stdout,"(at your option) any later version.\n");
 	fprintf(stdout,"You should have received a copy of the GNU General Public License\n");
	fprintf(stdout,"along with this program; if not, write to the Free Software\n");
	fprintf(stdout,"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
	fprintf(stdout,"(See the included file COPYING)\n\n");
}

void uso(void)
{
	fprintf(stdout,USO);
}

void outofbounds(int last)
{
	fprintf(stdout,OLTRE,last,last);
	if (!console)
		exit(1);
}

/* Pause or Resume ? */
void pause_resume (int cd_fd) {
	int status = cd_status (cd_fd);
	switch (status) {
		case CDROM_AUDIO_PLAY:
			cd_pause(cd_fd) ;
			break;
		case CDROM_AUDIO_PAUSED:
			cd_resume(cd_fd) ;
			break;
	}
}

/* Stop this music, please! */
void stop(int cd_fd)
{
	int status = cd_status (cd_fd);
	int arr_track[MAXTRACK];
	if (status==CDROM_AUDIO_PLAY || status==CDROM_AUDIO_PAUSED)
			cd_stop(cd_fd);
	arr_track[0]=0;
	writedata(cd_fd,0,arr_track,99,9);
}

/* Give me my CD back! */
void eject(int cd_fd)
{
	int status = cd_status (cd_fd);
	if (status!=CDROM_AUDIO_PLAY && status!=CDROM_AUDIO_PAUSED)
			cd_eject(cd_fd);
}

/* Set Volume */
void set_volume(int value)
{
	int mix_fd = mix_open(devmixer);
	if (mix_fd!=0) {
		mix_write(mix_fd, value);
		mix_close(mix_fd);
	}
}

/* Get Volume */
int get_volume(void)
{
	int mix_fd = mix_open(devmixer);
	int vlm=0;

	if (mix_fd!=0) {
		vlm=mix_read(mix_fd);
		mix_close(mix_fd);
	}
	return vlm;
}


void setdefault(char *defopt[],int last,int cd_fd,int nopt)
{
	FILE *fp,*fi;
	char *fcfg=malloc(100);
	char *fopt=malloc(255);
	char *ffil=malloc(255);
	char *title=malloc(100);
	char *s=malloc(200);
	int i,d=0;
	memset(title,0,100);
	gettitle(last,cd_fd,title);
	memset(fopt,0,255);
	snprintf(fcfg,100,"%s%s",getenv("HOME"),FILECFG);
	rename(fcfg,"/tmp/jac.config");
	fp=fopen(fcfg,"w");
	fi=fopen("/tmp/jac.config","r");
	if (fp==NULL || fi==NULL) {
		fprintf(stdout,ERRNOCFG,fcfg);
		return;
	}
	
	for (i=1;i<nopt;i++) {
		strcat(fopt,*(defopt+i));
		strcat(fopt," ");
	}

	while (fgets(s,200,fi)!=NULL) {
		snprintf(ffil,100,"%08lx",cddb_discid(last,cd_fd));
		if (strncmp(s,ffil,8)==0) {
			snprintf(ffil,255,"%08lx=%s\n",cddb_discid(last,cd_fd),fopt);
			fputs(ffil,fp);
			d=1;
			continue;
		}
		fputs(s,fp);
	}

	if (!d) {
		snprintf(ffil,255,"# Default option for: %s\n",title);
		fputs(ffil,fp);
		snprintf(ffil,255,"%08lx=%s\n",cddb_discid(last,cd_fd),fopt);
		fputs(ffil,fp);
	}
	
	fclose(fp);
	
}


/* Parse number from option list */
int getnum(char *s)
{
	int i,neg,a=0;
	char s1[5];
	
	memset(s1,0,5);
	neg=1;

	for (i=0;i<strlen(s) && a<=4;i++) {
		if (isdigit(s[i])) {
			s1[a++]=s[i];
			if (s[i-1]=='-' && a==1)
				neg=-1;
		} else {
			if (a>0)
				break;
		}
	}
	return atoi(s1)*neg;
}

/* Exclude track from playlist */
void leva(int n,int at[MAXTRACK])
{
	int i,a,nt=0;

	for (i=0;i<MAXTRACK && at[i]!=0;i++)
			nt++;

	for (i=0;i<nt;i++) {
		if (at[i]==n && n!=0) {
			for (a=i;a<nt;a++)
				at[a]=at[a+1];
			at[MAXTRACK-1]=0;
			nt--;
		}
	}
	at[nt]=0;
}

/* Get playlist from command line option */
int getseq(char *s,int at[MAXTRACK],int last,int startn,int metti)
{
	int i,n,a;
	char s1[5];
	for (a=0;a<5;a++) s1[a]='\0';
	n=a=0;

	for (i=startn;i<strlen(s);i++) {
		if (isdigit(s[i])) {
			s1[a++]=s[i];
			a=a>4 ? 4 : a;
		} else if (s[i]==44) {
			if (metti) {
				at[n]=atoi(s1);
				if (at[n++]>last) {
					outofbounds(last);
					return 1;
				}
			}
			else
				leva(atoi(s1),at);
			for (a=0;a<5;a++) s1[a]='\0';
			a=0;
		} else if (s[i]==61)
			continue;
		else
			break;
	}
	if (atoi(s1)>0) {
		if (metti) {
			at[n]=atoi(s1);
			if (at[n++]>last) {
				outofbounds(last);
				return 1;
			}
		} else
			leva(atoi(s1),at);
	}
	if (metti) {
		for (i=n;i<=last;i++)
			at[i]=0;
	}

	return 0;
}

/* Where are we, now? */
int getpos(cd *hw)
{
	int pos_abs,start;

	pos_abs = hw->subchnl.cdsc_absaddr.msf.minute * 60 +
		hw->subchnl.cdsc_absaddr.msf.second;
	start = hw->tocent[hw->subchnl.cdsc_trk-1].cdte_addr.msf.minute * 60 +
		hw->tocent[hw->subchnl.cdsc_trk-1].cdte_addr.msf.second;
	return pos_abs-start;

}

/* Duration in second of the selected (i) track */
int getsec(cd *hw,int i)
{
	int n;

	n=hw->tocent[i+1].cdte_addr.msf.minute * 60 +
		hw->tocent[i+1].cdte_addr.msf.second -
		hw->tocent[i].cdte_addr.msf.minute * 60 -
		hw->tocent[i].cdte_addr.msf.second; 
  
	return n;
}

int ncifre(int n)
{
	int i=1;
	while ((n/=10) != 0)
		i++;
	return i;
}

/* Jump np tracks - if np>0 skip fwd else skip back */
void jump(int cd_fd,int np,int at[MAXTRACK])
{
	int rp,crazy,nt,i;
	rp=crazy=0;
	i=gettracktoplay(FILEPARM,at,last_track(cd_fd),&rp,&crazy); 

	if (crazy==9 && i!=99) {
		nt=(i+np)-1;
		if (np>0)
			nt--;
		nt=nt<0 ? -1 : nt;	
		if (nt<last_track(cd_fd)) {
			if (at[nt+1]!=0 && !isdata(cd_fd,at[nt+1])) {
				writedata(cd_fd,rp,at,nt,9);
				cd_stop(cd_fd);
			}
		}
	}
}

/* Tell me everything about this CD */
void geninfo(int cd_fd,int inf,int ply)
{
	int i,first,last,nmin,nsec,pos,temp,rp,crazy,trk;
	int at[MAXTRACK];
	cd *hw=cd_init(cd_fd);
	FILE *fcddb;
	char tmp[255];

	first=hw->tochdr.cdth_trk0;
	last=hw->tochdr.cdth_trk1;
	fprintf(stdout,INFO,VERSION);
	fprintf(stdout,NBRANI,last);
	fprintf(stdout,DURTOT,hw->tocent[last].cdte_addr.msf.minute,hw->tocent[last].cdte_addr.msf.second);

	pos=getpos(hw);
	nsec = pos % 60;
	nmin = pos / 60;
	temp=0;
	rp=0;

	fprintf(stdout,STATUS);
	switch (hw->subchnl.cdsc_audiostatus) {
		case CDROM_AUDIO_PLAY:
			fprintf(stdout,PLAYING,hw->subchnl.cdsc_trk,nmin,nsec);
			temp=1;
			break;
		case CDROM_AUDIO_PAUSED:
			fprintf(stdout,PAUSED,hw->subchnl.cdsc_trk,nmin,nsec);
			temp=1;
			break;
		default:
			fprintf(stdout,INACTIVE);
			break;
	}

	fprintf(stdout,VOLLEVEL,get_volume());
	if (inf==2) {
		sprintf(tmp,"%s/%08lx",dircddb,cddb_discid(last,cd_fd));
		fcddb=fopen(tmp,"r");
		if (fcddb==NULL) {
			fprintf(stdout,NOCDDB);
			for (i=first-1;i<last;i++) {
				nsec=getsec(hw,i);
				nmin=nsec/60;
				nsec=nsec%60;            
				fprintf(stdout,DETTRACK,i+1,nmin,nsec);
				/*	
				if ( (((i+1)%3)==0) || i==last-1 )
				*/
				fprintf(stdout,"\n");
				/*
				else
					fprintf(stdout,"\t");
				*/
			}
		} else {
			while(fgets(tmp,80,fcddb)!=NULL) {
				tmp[strlen(tmp)-1] = 0;
				if (tmp[0]=='#')
					continue;
				if(strncmp(tmp,"DTITLE",6)==0) {
					fprintf(stdout,CDTITLE,tmp+7);
					continue;
				}
/*				if(strncmp(tmp,"TTITLE",6)==0) { */
				if(sscanf(tmp,"TTITLE%d=",&trk)==1) {
					nsec=getsec(hw,trk);
					nmin=nsec/60;
					nsec=nsec%60;            
					fprintf(stdout,CDTRACK,trk+1,
						((trk+1)==hw->subchnl.cdsc_trk && temp) ? "*" : " ",
						nmin,nsec,tmp+7+ncifre(trk));
				}
/*				} */
			}
		}

		if (!ply) {
			i=gettracktoplay(FILEPARM,at,last_track(cd_fd),&rp,&crazy); 
			if (i!=99 && tmp) {
				fprintf(stdout,PLAYLIST);
				for (i=0;i<MAXTRACK && at[i]!=0;i++) {
					fprintf(stdout,"%d ",at[i]);
				}
				fprintf(stdout,"\n");
				fprintf(stdout,"Repeat: %s\n",rp==0 ? "OFF" : "ON" ); 
				if (crazy!=9)
					fprintf(stdout,"Crazy : %s\n\n",crazy==0 ? "OFF" : "ON" ); 
			}
		}
	}
}


/* Plays playlist (included repeat) */                     
int playlist(int repeat,int cd_fd,int arr_track[MAXTRACK])
{
	int i,wait,last,crazy;
	int status = cd_status (cd_fd);


	/* If CD is playing I will stop it, and start playing myself */
	if (status==CDROM_AUDIO_PLAY) {
		writedata(cd_fd,0,arr_track,99,9);
		cd_stop(cd_fd);
		sleep(3);
	}

	writedata(cd_fd,repeat,arr_track,0,9);
	i=0;
	last=last_track(cd_fd);
	while (isdata(cd_fd,arr_track[i]))
		i++;
	
	while (i!=99) {
		cd_play(cd_fd,arr_track[i]);
		sleep(5);
		wait=1;
		while (wait) {
		 	switch (cd_status(cd_fd)) {
				case CDROM_AUDIO_PLAY:
				case CDROM_AUDIO_PAUSED:
					sleep(2);
					break;
				default:
					i=gettracktoplay(FILEPARM,arr_track,last,&repeat,&crazy); 
					wait=0;
					break;
			}
			if (i!=99) {
				if (arr_track[i]!=0) {
					if (isdata(cd_fd,arr_track[i])) {
						writedata(cd_fd,repeat,arr_track,i,9);
						wait=1;
					}
				}
			}
		}
		if (i!=99) {
			if (arr_track[i]==0) {
				if (repeat)
					i=0;
				else
					i=99;
			}
		}
		writedata(cd_fd,repeat,arr_track,i,9);
	}
	return 0;
}

/* Random sorting of the playlist. Randomization based on PID */
void randsort(int arr_track[MAXTRACK])
{
	int i,nt,n,at[MAXTRACK];
	
	for (i=0;i<MAXTRACK;i++) {
		if (arr_track[i]==0)
			break;
		at[i]=arr_track[i];
	}

	/*srand((int) getpid());*/
	nt=i;
	while (i>0) {
		n=rand() % nt;
		if (at[n]!=0) {
			arr_track[nt-i]=at[n];
			at[n]=0;
			i--;
		}
	}
}

